"""Data models corresponding to the tables under the ODM2Annotations schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Annotations.html
"""

class ActionAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_ActionAnnotations.html"""

class Annotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_Annotations.html"""

class CategoricalResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_CategoricalResultValueAnnotations.html"""

class EquipmentAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_EquipmentAnnotations.html"""

class MethodAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_MethodAnnotations.html"""

class PointCoverageResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_PointCoverageEesultValueAnnotations.html"""

class ProfileResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_ProfileResultValueAnnotations.html"""

class ResultAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_ResultAnnotations.html"""

class SamplingFeatureAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_SamplingFeatureAnnotations.html"""

class SectionResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_SectionResultValueAnnotations.html"""

class SpectraResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_SpectraResultValueAnnotations.html"""

class TimeSeriesResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_TimeSeriesResultValueAnnotations.html"""

class TrajectoryResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_TrajectoryResultValueAnnotations.html"""

class TransectResultValueAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Annotations_TransectResultValueAnnotations.html"""


"""Data models corresponding to the tables under the ODM2Equipment schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Equipment.html
"""

class CalibrationActions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_CalibrationActions.html"""

class CalibrationReferenceEquipment():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_CalibrationReferenceEquipment.html"""

class CalibrationStandards():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_CalibrationStandards.html"""

class DataloggerFileColumns():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_DataloggerFileColumns.html"""

class DataLoggerFiles():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_DataLoggerFiles.html"""

class DataLoggerProgramFiles():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_DataloggerProgramFiles.html"""

class Equipment():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_Equipment.html"""

class EquipmentModels():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_EquipmentModels.html"""

class EquipmentUsed():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_EquipmentUsed.html"""

class InstrumentOutputQuantityKind():
	""""""

class MaintenanceActions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_MaintenanceActions.html"""

class RelatedEquipment():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Equipment_RelatedEquipment.html"""


"""Data models corresponding to the tables under the ODM2Core schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Core.html
"""

class ActionBy():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_ActionBy.html"""

class Actions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Actions.html"""

class Affiliations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Affiliations.html"""

class Datasets():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Datasets.html"""

class DatasetsResults():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_DatasetsResults.html"""

class FeatureActions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_FeatureActions.html"""

class Methods():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Methods.html"""

class Organizations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Organizations.html"""

class People():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_People.html"""

class ProcessingLevels():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_ProcessingLevels.html"""

class RelatedActions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_RelatedActions.html"""

class Results():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Results.html"""

class SamplingFeatures():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_SamplingFeatures.html"""

class TaxonomicClassifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_TaxonomicClassifiers.html"""

class CV_Units():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Units.html"""

class Variables():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Core_Variables.html"""


"""Data models corresponding to the tables under the ODM2Simulation schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Simulation.html
"""

class ModelAffiliations ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Simulation_ModelAffiliations.html"""

class Models ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Simulation_Models.html"""

class RelatedModels ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Simulation_RelatedModels.html"""

class Simulations ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Simulation_Simulations.html"""


"""Data models corresponding to the tables under the ODM2LabAnalyses schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2LabAnalyses.html
"""

class ActionDirectives():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2LabAnalyses_ActionDirectives.html"""

class Directives():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2LabAnalyses_Directives.html"""

class SpecimenBatchPositions():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2LabAnalyses_SpecimenBatchPostions.html"""


from . import annotations
from . import auth
from . import core
from . import cv
from . import dataquality
from . import equipment
from . import extensionproperties
from . import externalidentifiers
from . import labanalyses
from . import provenance
from . import results
from . import samplingfeatures
from . import simulation


"""Data models corresponding to the tables under the ODM2CV schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2CV.html
"""

class CV_ActionType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_ActionType.html"""

class CV_AggregationStatistic():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_AggregationStatistic.html"""

class CV_AnnotationType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_AnnotationType.html"""

class CV_CensorCode():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_CensorCode.html"""

class CV_DataQualityType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_DataQualityType.html"""

class CV_DatasetType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_DatasetType.html"""

class CV_DirectiveType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_DirectiveType.html"""

class CV_ElevationDatum():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_ElevationDatum.html"""

class CV_EquipmentType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_EquipmentType.html"""

class CV_FeaturesOfInterestType():
	""""""

class CV_LBNLICPMSMapper():
	""""""

class CV_Medium():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_Medium.html"""

class CV_MethodType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_MethodType.html"""

class CV_OrganizationType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_OrganizationType.html"""

class CV_PropertyDataType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_PropertyDataType.html"""

class CV_QualityCode():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_QualityCode.html"""

class CV_QuantityKind():
	""""""

class CV_RelationshipType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_RelationshipType.html"""

class CV_ResultType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_ResultType.html"""

class CV_SamplingFeatureGeoType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_SamplingFeatureGeoType.html"""

class CV_SamplingFeatureType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_SamplingFeatureType.html"""

class CV_SpatialOffsetType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_SpatialOffsetType.html"""

class CV_SpecimenCollection():
	""""""

class CV_SpecimenType():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_SpecimenType.html"""

class CV_Status():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2CV_CV_Status.html"""

class CV_TaxonomicClassifierDomain():
	""""""

class CV_Units():
	""""""

class CV_VariableDomain():
	""""""



"""Data models corresponding to the tables under the ODM2ExtensionProperties schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2ExtensionProperties.html
"""

class ActionExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_ActionExtensionPropertyValues.html"""

class CitationExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_CitationExtensionPropertyValues.html"""

class ExtensionProperties():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_ExtensionProperties.html"""

class MethodExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_MethodExtensionPropertyValues.html"""

class ResultExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_ResultExtensionPropertyValues.html"""

class SamplingFeatureExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_SamplingFeatureExtensionPropertyValues.html"""

class VariableExtensionPropertyValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExtensionProperties_VariableExtensionPropertyValues.html"""



"""Data models corresponding to the tables under the ODM2DataQuality schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2DataQuality.html
"""

class DataQuality():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2DataQuality_DataQuality.html"""

class ReferenceMaterials():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2DataQuality_ReferenceMaterials.html"""

class ReferenceMaterialValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2DataQuality_ReferenceMaterialValues.html"""

class ResultNormalizationValues():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2DataQuality_ResultNormalizationValues.html"""

class ResultsDataQuality():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2DataQuality_ResultsDataQuality.html"""


import sqlalchemy
from sqlalchemy.sql.expression import Select
from sqlalchemy.orm import Query    
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.ext.declarative import declared_attr, declarative_base
import geoalchemy2

import pickle
from enum import Enum
from typing import Dict, Union, Any, Type
import warnings

import pandas as pd

from .exceptions import ObjectNotFound

from .models import annotations
from .models import auth
from .models import core
from .models import cv
from .models import dataquality
from .models import equipment
from .models import extensionproperties
from .models import externalidentifiers
from .models import labanalyses
from .models import provenance
from .models import results
from .models import samplingfeatures
from .models import simulation

import warnings
warnings.simplefilter("ignore", category=sqlalchemy.exc.SAWarning)

OUTPUT_FORMATS = ('json', 'dataframe', 'dict')



class Base():
    
    @declared_attr
    def __tablename__(self) -> str:
        cls_name = str(self.__name__)
        return cls_name.lower()

    @classmethod
    def from_dict(cls, attributes_dict:Dict) -> object:
        """Alternative constructor that uses dictionary to populate attributes"""
        instance = cls.__new__(cls)
        instance.__init__()
        for key, value in attributes_dict.items():
            if hasattr(instance, key):
                if value == '': value = None
                setattr(instance, key, value)
        return instance

    def to_dict(self) -> Dict[str,Any]:
        """Converts attributes into a dictionary"""
        columns = self.__table__.columns.keys()
        output_dict = {}
        for column in columns:
            output_dict[column] = getattr(self,column)
        return output_dict
    
    def update_from_dict(self, attributes_dict:Dict[str, any]) -> None:
        """Updates instance attributes based on provided dictionary"""
        for key, value in attributes_dict.items():
            if hasattr(self, key):
                if value == '': value = None
                setattr(self, key, value)

    @classmethod
    def get_pkey_name(cls) -> Union[str,None]:
        """ Returns the primary key field name for a given model"""
        columns = cls.__table__.columns
        for column in columns:
            if column.primary_key: return column.name 
        return None

class ODM2Engine:

    def __init__(self, session_maker:sqlalchemy.orm.sessionmaker) -> None:
        self.session_maker = session_maker

    def read_query(self, 
            query: Union[Query, Select],
            output_format:str='json',
            orient:str='records') -> Union[str, pd.DataFrame]:

        # guard against invalid output_format strings
        if output_format not in OUTPUT_FORMATS:
            raise ValueError(f':argument output_format={output_format}, is not a valid output_format strings: {OUTPUT_FORMATS}')
        
        # use SQLAlchemy session to read_query and return response in the designated output_format
        with self.session_maker() as session:
            if isinstance(query, Select):
                df = pd.read_sql(query, session.bind)
            else:
                df = pd.read_sql(query.statement, session.bind)
            
            if output_format == 'json':
                return df.to_json(orient=orient)
            elif output_format == 'dataframe':
                return df
            elif output_format == 'dict':
                return df.to_dict()
            raise TypeError("Unknown output format")

    def insert_query(self) -> None:
        """Placeholder for bulk insert"""
        #accept dataframe & model
        #use pandas to_sql method to perform insert
        #if except return false or maybe raise error 
        #else return true
        raise NotImplementedError

    def create_object(self, obj:object) -> Union[int, str]:
        pkey_name = obj.get_pkey_name()
        setattr(obj, pkey_name, None)

        with self.session_maker() as session:
            session.add(obj)
            session.commit()
            pkey_value = getattr(obj, pkey_name)
            return pkey_value

    def read_object(self, model:Type[Base], pkey:Union[int, str], 
            output_format:str='dict', 
            orient:str='records') -> Dict[str, Any]:

        with self.session_maker() as session:
            obj = session.get(model, pkey)
            pkey_name = model.get_pkey_name()
            if obj is None: raise ObjectNotFound(f"No '{model.__name__}' object found with {pkey_name} = {pkey}")
            session.commit()

            # convert obj_dict to a dictionary if it isn't one already
            obj_dict = obj.to_dict()

            # guard against invalid output_format strings
            if output_format not in OUTPUT_FORMATS:
                raise ValueError(f':param output_format = {output_format}, which is not one of the following valid output_format strings: {OUTPUT_FORMATS}')

            if output_format == 'dict':
                return obj_dict

            else:
                # convert to series if only one row
                keys = list(obj_dict.keys())
                if not isinstance(obj_dict[keys[0]], list):
                    for key in keys:
                        new_value = [obj_dict[key]]
                        obj_dict[key] = new_value

                obj_df = pd.DataFrame.from_dict(obj_dict)
                if output_format == 'dataframe':
                    return obj_df
                elif output_format == 'json':
                    return obj_df.to_json(orient=orient)
                raise TypeError("Unknown output format")


    def update_object(self, model:Type[Base], pkey:Union[int,str], data:Dict[str, Any]) -> None:
        if not isinstance(data, dict):
            data = data.dict()
        pkey_name = model.get_pkey_name()
        if pkey_name in data:
            data.pop(pkey_name)
        with self.session_maker() as session:
            obj = session.get(model, pkey)
            if obj is None: raise ObjectNotFound(f"No '{model.__name__}' object found with {pkey_name} = {pkey}")
            obj.update_from_dict(data)
            session.commit()

    def delete_object(self, model:Type[Base], pkey:Union[int, str]) -> None:
        with self.session_maker() as session:
            obj = session.get(model, pkey)
            pkey_name = model.get_pkey_name()
            if obj is None: raise ObjectNotFound(f"No '{model.__name__}' object found with {pkey_name} = {pkey}")
            session.delete(obj)
            session.commit()

class Models:

    def __init__(self, base_model) -> None:
        self._base_model = base_model
        self._process_schema(annotations)        
        self._process_schema(auth)        
        self._process_schema(core)        
        self._process_schema(cv)        
        self._process_schema(dataquality)        
        self._process_schema(equipment)        
        self._process_schema(extensionproperties)        
        self._process_schema(externalidentifiers)        
        self._process_schema(labanalyses)        
        self._process_schema(provenance)        
        self._process_schema(results)        
        self._process_schema(samplingfeatures)        
        self._process_schema(simulation)         

    def _process_schema(self, schema:str) -> None:
        classes = [c for c in dir(schema) if not c.startswith('__')]
        base = tuple([self._base_model])
        for class_name in classes:
            model = getattr(schema, class_name)
            model_attribs = self._trim_dunders(dict(model.__dict__.copy())) 
            extended_model =  type(class_name, base, model_attribs) 
            setattr(self, class_name, extended_model)

    def _trim_dunders(self, dictionary:Dict[str, Any]) -> Dict[str, Any]:
        return { k:v for k, v in dictionary.items() if not k.startswith('__') } 

class ODM2DataModels():

    def __init__(self, engine:sqlalchemy.engine, schema:str='odm2', cache_path:str=None) -> None:

        self._schema = schema
        self._cache_path = cache_path

        self._engine = engine
        self._session = sqlalchemy.orm.sessionmaker(self._engine)
        self._cached= False
        self.odm2_engine: ODM2Engine = ODM2Engine(self._session)

        self._model_base = self._prepare_model_base()
        self.models = Models(self._model_base)
        if not self._cached:
            self._prepare_automap_models()

    def _prepare_model_base(self):
        try:
            with open(self._cache_path, 'rb') as file:
                metadata = pickle.load(file=file)
                self._cached = True
                return declarative_base(cls=Base, bind=self._engine, metadata=metadata)
        except FileNotFoundError: 
            metadata = sqlalchemy.MetaData(schema=self._schema)
            self._cached = False
            return automap_base(cls=Base, metadata=metadata)

    def _prepare_automap_models(self):
        self._model_base.prepare(self._engine)
        if not self._cache_path: return
        try:
            with open(self._cache_path, 'wb') as file:
                pickle.dump(self._model_base.metadata, file)
        except FileNotFoundError:
            warnings.warn('Unable to cache models which may lead to degraded performance.', RuntimeWarning)


"""
"""

class Accounts():
    """"""

#PRT - even though this is a CV table it is for account auth so we might move this to the auth module.
class CV_Permission():
	""""""

class OrganizationsPermissions():
    """"""

class OrganizationsSamplingFeatures():
    """"""

class ResultsPermissions():
    """"""

class Roles():
    """"""

class SamplingFeaturesPermissions():
    """"""  



"""Data models corresponding to the tables under the ODM2SamplingFeatures schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2SamplingFeatures.html
"""

class FeaturesOfInterest():
	""""""

class RelatedFeatures ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2SamplingFeatures_RelatedFeatures.html"""

class SampledFeatures():
	""""""

class SpatialOffsets ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2SamplingFeatures_SpatialOffsets.html"""

class SpatialReferences ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2SamplingFeatures_SpatialReferences.html"""

class SpecimenCollections ():
	""""""	

class Specimens ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2SamplingFeatures_Specimens.html"""

class SpecimenTaxonomicClassifiers ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2SamplingFeatures_SpecimenTaxonomicClassifiers.html"""


"""Data models corresponding to the tables under the ODM2Core schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2ExternalIdentifiers.html
"""

class CitationExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_CitationExternalIdentifiers.html"""

class ExternalIdentifierSystems():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_ExternalIdentifierSystems.html"""

class MethodExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_MethodExternalIdentifiers.html"""

class PersonExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_PersonExternalIdentifiers.html"""

class ReferenceMaterialExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_ReferenceMaterialExternalIdentifiers.html"""

class SamplingFeatureExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_SamplingFeatureExternalIdentifiers.html"""

class SpatialReferenceExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_SpatialReferenceExternalIdentifiers.html"""

class TaxonomicClassifierExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_TaxonomicClassifierExternalIdentifiers.html"""

class VariableExternalIdentifiers():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2ExternalIdentifiers_VariableExternalIdentifiers.html"""


"""Data models corresponding to the tables under the ODM2Results schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Results.html
"""

class CategoricalResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_CategoricalResults.html"""

class CategoricalResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_CategoricalResultValues.html"""

class MeasurementResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_MeasurementResults.html"""

class MeasurementResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_MeasurementResultValues.html"""

class PointCoverageResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_PointCoverageResults.html"""

class PointCoverageResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_PointCoverageResultValues.html"""

class ProfileResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_ProfileResults.html"""

class ProfileResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_ProfileResultValues.html"""

class SectionResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_SectionResults.html"""

class SectionResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_SectionResultValues.html"""

class SpectraResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_SpectraResults.html"""

class SpectraResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_SpectraResultValues.html"""

class TimeSeriesResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TimeSeriesResults.html"""

class TimeSeriesResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TimeSeriesResultValues.html"""

class TrajectoryResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TrajectoryResults.html"""

class TrajectoryResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TrajectoryResultValues.html"""

class TransectResults ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TransectResults.html"""

class TransectResultValues ():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Results_TransectResultValues.html"""


"""Data models corresponding to the tables under the ODM2Provenance schema
	Reference: http://odm2.github.io/ODM2/schemas/ODM2_Current/schemas/ODM2Provenance.html
"""

class AuthorLists():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_AuthorLists.html"""

class Citations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_Citations.html"""

class DataSetCitations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_DatasetCitations.html"""

class DerivationEquations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_DerivationEquations.html"""

class MethodCitations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_MethodCitations.html"""
	
class RelatedAnnotations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_RelatedAnnotations.html"""

class RelatedDatasets():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_RelatedDatasets.html"""

class RelatedResults():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_RelatedResults.html"""

class ResultDerivationEquations():
	"""http://odm2.github.io/ODM2/schemas/ODM2_Current/tables/ODM2Provenance_ResultDerivationEquations.html"""